/**
 * Title: Painter - Application implementing the Printable interface (see MyPrinter)
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.base ;

import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.print.*;
import java.text.*;

public class Painter implements Printable {

	static AttributedString myText;

	// FIRST CONSTRUCTOR (not used)
	public Painter() {

		myText = new AttributedString("Default text");
	}

	// SECOND CONSTRUCTOR (invoked by the MyPrinter class)
	public Painter(String text) {

		myText = new AttributedString(text);
	}

	// MAIN METHOD
	public int print(Graphics graphic, PageFormat format, int pageIndex) {

		Graphics2D g2d = (Graphics2D) graphic;

		// Move the origin from the corner of the Paper to the corner of the imageable area.
		g2d.translate(format.getImageableX(), format.getImageableY());
		g2d.setPaint(Color.black);

		// LineBreakMeasurer will break the text into lines that fit the imageable area.
		Point2D.Float pen = new Point2D.Float();
		AttributedCharacterIterator charIterator = myText.getIterator();
		LineBreakMeasurer measurer = new LineBreakMeasurer(charIterator, g2d.getFontRenderContext());
		float wrappingWidth = (float) format.getImageableWidth();

		while (measurer.getPosition() < charIterator.getEndIndex()) {

			TextLayout layout = measurer.nextLayout(wrappingWidth);
			pen.y += layout.getAscent();
			float dx = layout.isLeftToRight()? 0 : (wrappingWidth - layout.getAdvance());

			layout.draw(g2d, pen.x + dx, pen.y);
			pen.y += layout.getDescent() + layout.getLeading();

		}

		return Printable.PAGE_EXISTS ;		//alternative:	return NO_SUCH_PAGE ;
	}

} // End of class
