/**
 * Title: Sample 12 - A very simple videogame: "catch the black spot"
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.base ;

import java.applet.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;

public class Sample12 extends Applet implements ActionListener, MouseMotionListener, MouseListener {

	Date clock ;
	long time ;
	Graphics graphic = getGraphics() ;

	Label heading    = new Label("CATCH THE BLACK OVAL") ;
	TextArea help    = new TextArea("Goal of this game is to catch with the mouse one black " +
	"oval running on the screen.\nTo win you must click ON the black dot, you can change " +
	"the difficulty of the game by pressing: \n \n ' - '        (on the left)   = EASIER \n" +
	" ' + '       (on the right) = HARDER \n\nTo play press 'Start'.",2,10) ;

    Button pressMe  = new Button("Start") ;
    Button harder   = new Button("+") ;
    Button easier   = new Button("-") ;

	int sizeX = 20 ; int sizeY = 10 ;	// the oval sizes
    int x0 = 20 ;    int y0 = 25 ;		// margins defining inner area where oval moves
    int x, y, maxX, maxY, lastX, lastY ;

    boolean running = false ;	// this flag indicates when player pressed "Start"
    boolean catched = false ;	// this flag indicates if the player won the game
    long delay = 500 ;    		// default delay for the random movement of the black dot (msec)

    public void init() {

		setLayout(new BorderLayout()) ;
		heading.setAlignment(Label.CENTER) ;

        add(heading, "North") ;
        add(pressMe, "South") ;
        add(easier,"West") ;
        add(harder,"East") ;
        add(help,"Center") ;

        pressMe.addActionListener(this) ;
        easier.addActionListener(this) ;
        harder.addActionListener(this) ;
        addMouseMotionListener(this) ;
        addMouseListener(this) ;

		// Calling getSize() on the applet itself (this)
		maxX =  getSize().width ;
		maxY =  getSize().height ;
		graphic = getGraphics() ;
		graphic.setFont(new Font("Helvetica", Font.PLAIN, 12)) ;
 	    graphic.setColor(Color.BLACK) ;

 		x = (int) maxX/2 ;    		// calculates initial position of the black oval
 		y = (int) maxY/2 ;
 	    lastX = maxX - 2*x0 ;		// calculates size of inner area where oval moves
 	    lastY = maxY - 2*y0 ;
    }

    public void actionPerformed(ActionEvent event) {

		char action = event.getActionCommand().charAt(0) ;

		switch (action) {
			case 'S': remove(help) ;			// remove the TextArea from the "Center" position
				 	  catched = false ;
					  running = true;
					  clock = new Date() ;
					  time = clock.getTime() ;	// set the Class attribute "time" (see below)
					  break ;
			case '-': delay += 50 ;
					  break ;
			case '+': if (delay > 100) delay -= 50 ;
					  break ;
		}

		heading.setText("Time: " + delay + " ms") ;	// shows the difficulty's value selected
	}

	public void mouseMoved(MouseEvent event) {

		clock = new Date() ;
		long now = clock.getTime() ;
		boolean moveNow = (now - time) > delay ; // Moves the oval only after "delay" milliseconds!

        if ( (running) && (moveNow) ) {

		   time = now ;		// stores timestamp of the last movement of the oval

 	       Random dice = new Random() ;
	       Double xx = new Double(lastX*(dice.nextDouble())) ;
 	       Double yy = new Double(lastY*(dice.nextDouble())) ;

		   // Remark: the coordinates (x,y) are not the CENTER of the oval !
	       graphic.clearRect(x, y, sizeX, sizeY) ;	// clears previous black oval
 	       x = xx.intValue() ;
 	       y = yy.intValue() ;
 	       graphic.fillOval(x, y, sizeX, sizeY) ;		// draws the new black oval
		}
    }

    public void mouseClicked(MouseEvent event) {

            int x1 = event.getX() ;
            int y1 = event.getY() ;
            int deltaX = Math.abs(x1 - x - sizeX/2) ;  // distance from the center of the black dot
            int deltaY = Math.abs(y1 - y) ;
            double squared = deltaX*deltaX + deltaY*deltaY ;
            double distance = Math.sqrt(squared) ;

            if ((distance < sizeX) && running) {
                heading.setText("CATCHED !!!") ;
                catched = true ;
            }
                else heading.setText("FAILED ...") ;

			running = false ;		// game ends the first time the player tries to click !!!
     }

	// Methods not implemented:
    public void mouseDragged(MouseEvent event)  { }
	public void mouseEntered(MouseEvent event)  { }
    public void mouseExited(MouseEvent event)   { }
    public void mousePressed(MouseEvent event)  { }
    public void mouseReleased(MouseEvent event) { }

} // End of class